package de.intevation.artifactdatabase.rest;

import java.io.IOException;

import org.apache.log4j.Logger;
import org.restlet.data.MediaType;
import org.restlet.data.Response;
import org.restlet.data.Status;
import org.restlet.ext.xml.DomRepresentation;
import org.restlet.representation.EmptyRepresentation;
import org.restlet.representation.Representation;
import org.restlet.resource.ResourceException;
import org.w3c.dom.Document;

import de.intevation.artifactdatabase.XMLUtils;
import de.intevation.artifacts.ArtifactDatabase;
import de.intevation.artifacts.ArtifactDatabaseException;
import de.intevation.artifacts.ArtifactNamespaceContext;

/**
 * @author <a href="mailto:sascha.teichmann@intevation">Sascha L. Teichmann</a>
 */
public class CreateResource
extends      BaseResource
{
    private static Logger logger = Logger.getLogger(CreateResource.class);

    public static final String PATH = "/create";

    public static final String XPATH_FACTORY = "/art:action/art:factory/@name";

    public static final String NO_FACTORY_MESSAGE = "No factory given";

    public static final String NO_ARTIFACT_CREATED = "No artifact created";

    protected Representation innerPost(Representation requestRepr)
    throws ResourceException
    {
        Document inputDocument = null;
        try {
            DomRepresentation input = new DomRepresentation(requestRepr);
            input.setNamespaceAware(true);
            inputDocument = input.getDocument();
        }
        catch (IOException ioe) {
            logger.error(ioe.getMessage());
            Response response = getResponse();
            response.setStatus(Status.CLIENT_ERROR_BAD_REQUEST, ioe);
            return new EmptyRepresentation();
        }

        String factory = XMLUtils.xpathString(
            inputDocument,
            XPATH_FACTORY,
            ArtifactNamespaceContext.INSTANCE);

        if (factory == null || factory.length() == 0) {
            Response response = getResponse();
            response.setStatus(
                Status.CLIENT_ERROR_BAD_REQUEST, NO_FACTORY_MESSAGE);
            return new EmptyRepresentation();
        }

        if (logger.isDebugEnabled()) {
            logger.debug("Create artifact with factory '" + factory + "'");
        }

        ArtifactDatabase db = (ArtifactDatabase)getContext()
            .getAttributes().get("database");

        try {
            return new DomRepresentation(
                MediaType.APPLICATION_XML,
                db.createArtifactWithFactory(factory, 
                                             getCallMeta(),
                                             inputDocument));
        }
        catch (ArtifactDatabaseException adbe) {
            Response response = getResponse();
            response.setStatus(
                Status.CLIENT_ERROR_UNPROCESSABLE_ENTITY, adbe.getMessage());
            return new EmptyRepresentation();
        }
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8:
